package controller

import (
	"electricity_bill_calc/model"
	"electricity_bill_calc/response"
	"electricity_bill_calc/security"
	"electricity_bill_calc/service"
	"net/http"

	"github.com/gofiber/fiber/v2"
)

func InitializeStatisticsController(router *fiber.App) {
	router.Get("/audits", security.OPSAuthorize, currentAuditAmount)
	router.Get("/stat/reports", security.MustAuthenticated, statReports)
}

func currentAuditAmount(c *fiber.Ctx) error {
	result := response.NewResult(c)
	amount, err := service.WithdrawService.AuditWaits()
	if err != nil {
		return result.Error(http.StatusInternalServerError, err.Error())
	}
	return result.Json(http.StatusOK, "已经获取到指定的统计信息。", fiber.Map{
		"amounts": map[string]int64{
			"withdraw": amount,
		},
	})
}

func statReports(c *fiber.Ctx) error {
	result := response.NewResult(c)
	session, err := _retreiveSession(c)
	if err != nil {
		return result.Unauthorized(err.Error())
	}
	var (
		enterprises int64 = 0
		parks       int64 = 0
		reports     []model.ParkPeriodStatistics
	)
	if session.Type != 0 {
		enterprises, err = service.StatisticsService.EnabledEnterprises()
		if err != nil {
			return result.Error(http.StatusInternalServerError, err.Error())
		}
		parks, err = service.StatisticsService.EnabledParks()
		if err != nil {
			return result.Error(http.StatusInternalServerError, err.Error())
		}
		reports, err = service.StatisticsService.ParksNewestState()
		if err != nil {
			return result.Error(http.StatusInternalServerError, err.Error())
		}
	} else {
		parks, err = service.StatisticsService.EnabledParks(session.Uid)
		if err != nil {
			return result.Error(http.StatusInternalServerError, err.Error())
		}
		reports, err = service.StatisticsService.ParksNewestState(session.Uid)
		if err != nil {
			return result.Error(http.StatusInternalServerError, err.Error())
		}
	}

	return result.Json(http.StatusOK, "已经完成园区报告的统计。", fiber.Map{
		"statistics": fiber.Map{
			"enterprises": enterprises,
			"parks":       parks,
			"reports":     reports,
		},
	})
}
